/*
 * CachingWithAspectJ
 * Copyright (C) 2008 Christian Schenk
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package org.christianschenk.aspectj.caching.helper;

import org.aspectj.lang.JoinPoint;

/**
 * Some helper methods for AspectJ.
 * 
 * @author Christian Schenk
 */
public class AspectJHelper {

	/**
	 * Convenience method that returns the class- and method-name for a given join point.
	 */
	public static final String getJoinPointName(final JoinPoint joinPoint) {
		return joinPoint.getThis().getClass().getSimpleName() + "." + joinPoint.getSignature().getName();
	}

	/**
	 * Returns the arguments of the current join point as a string.
	 * @param joinPoint
	 * @return string representing the arguments of this join point
	 */
	public static final String getJointPointArgs(final JoinPoint joinPoint) {
		final StringBuilder buf = new StringBuilder();
		for (final Object arg : joinPoint.getArgs()) {
			buf.append(arg.getClass().getSimpleName() + "-" + arg + "+");
		}
		return buf.toString().replaceAll("\\+$", "");
	}
}