/*
 * CachingWithAspectJ
 * Copyright (C) 2008 Christian Schenk
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package org.christianschenk.aspectj.caching.example;

import java.util.Random;

import org.apache.log4j.Logger;
import org.christianschenk.aspectj.caching.exampleapplication.MyApplication;
import org.christianschenk.common.StringUtils;
import org.christianschenk.stopwatch.StopWatch;
import org.junit.Before;
import org.junit.Test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

public class MyApplicationTest {

	private static final Logger log = Logger.getLogger(MyApplicationTest.class);
	private MyApplication myApp;

	@Before
	public void setup() {
		this.myApp = new MyApplication();
	}

	// generate cache misses
	@Test
	public void cacheMiss() {
		StopWatch.start();
		for (int i = 1; i <= 4; i++) {
			assertEquals(i * 50, this.myApp.getExpensiveObject(i).getParameter());
			assertEquals(i * 45, this.myApp.getAnotherExpensiveObject(i, (long) i).getParameter());
		}
		StopWatch.end();
		assertTrue(StopWatch.getResult() > 1000 && StopWatch.getResult() < 1200);
	}

	// generate cache hits
	@Test
	public void cacheHits() {
		StopWatch.start();
		for (int i = 1; i <= 200; i++) {
			assertEquals(250, this.myApp.getExpensiveObject(5).getParameter());
			assertEquals(225, this.myApp.getAnotherExpensiveObject(5, (long) 2).getParameter());
		}
		StopWatch.end();
		assertTrue(StopWatch.getResult() > 500 && StopWatch.getResult() < 600);
	}

	// testing random access
	@Test
	public void random() {
		final Random rand = new Random();

		StopWatch.start();
		int timeWithoutCache = 0;
		for (int i = 0; i < 100000; i++) {
			final int nextTime = rand.nextInt(1000) % 10;
			timeWithoutCache += nextTime;
			this.myApp.getExpensiveObject(nextTime);
		}
		StopWatch.end();

		log.info("Time with    cache: " + StopWatch.getResult() + " sec.");
		log.info("Time without cache: " + timeWithoutCache + " sec.");
		log.info("Ratio: " + StringUtils.formatNumber(1 - (double) StopWatch.getResult() / (double) timeWithoutCache, 2, 4) + " %");
	}
}